module beamdata
  use const_and_precisions, only : wp_
  implicit none

  integer, parameter :: jmx=31,kmx=36,nmx=100000
  integer, save :: nrayr,nrayth,nstep
  real(wp_), dimension(:,:,:), allocatable, save :: psjki,ppabs,ccci,tauv,alphav
  real(wp_), dimension(:,:,:), allocatable, save :: pdjki,currj,didst
  integer, dimension(:,:), allocatable, save :: iiv,iop,iow,ihcd,istore
  real(wp_), dimension(:,:), allocatable, save :: tau1v
  real(wp_), dimension(:), allocatable, save :: q
  real(wp_), dimension(:,:,:), allocatable, save :: yyrfl !(:,:,6)
  real(wp_), dimension(:,:,:), allocatable, save :: ywrk,ypwrk !(6,:,:)
  real(wp_), dimension(:,:,:), allocatable, save :: xc,xco,du1,du1o !(3,:,:)
  real(wp_), dimension(:,:,:), allocatable, save :: gri,dgrad2v !(3,:,:)
  real(wp_), dimension(:,:,:,:), allocatable, save :: ggri !(3,3,:,:)
  real(wp_), dimension(:,:), allocatable, save :: grad2
  real(wp_), dimension(:), allocatable, save :: dffiu,ddffiu
  complex(wp_), dimension(:,:,:), allocatable, save :: ext,eyt

contains

  subroutine alloc_beam(ierr)
    implicit none
    integer, intent(out) :: ierr
    
    call dealloc_beam
    allocate(psjki(nrayr,nrayth,nstep),  ppabs(nrayr,nrayth,nstep), &
             pdjki(nrayr,nrayth,nstep),   ccci(nrayr,nrayth,nstep), &
             currj(nrayr,nrayth,nstep),  didst(nrayr,nrayth,nstep), &
              tauv(nrayr,nrayth,nstep), alphav(nrayr,nrayth,nstep), &
                     iiv(nrayr,nrayth),          iop(nrayr,nrayth), &
                     iow(nrayr,nrayth),        tau1v(nrayr,nrayth), &
                    ihcd(nrayr,nrayth),       istore(nrayr,nrayth), &
                              q(nrayr),      yyrfl(nrayr,nrayth,6), &
                  ywrk(6,nrayr,nrayth),      ypwrk(6,nrayr,nrayth), &
                    xc(3,nrayr,nrayth),        xco(3,nrayr,nrayth), &
                   du1(3,nrayr,nrayth),       du1o(3,nrayr,nrayth), &
                   gri(3,nrayr,nrayth),    dgrad2v(3,nrayr,nrayth), &
                ggri(3,3,nrayr,nrayth),        grad2(nrayr,nrayth), &
                          dffiu(nrayr),              ddffiu(nrayr), &
                 ext(nrayr,nrayth,0:4),      eyt(nrayr,nrayth,0:4), &
                             stat=ierr)
    if (ierr/=0) call dealloc_beam
  end subroutine alloc_beam

  subroutine dealloc_beam
    implicit none
    if (allocated(psjki))   deallocate(psjki)
    if (allocated(ppabs))   deallocate(ppabs)
    if (allocated(pdjki))   deallocate(pdjki)
    if (allocated(ccci))    deallocate(ccci)
    if (allocated(currj))   deallocate(currj)
    if (allocated(didst))   deallocate(didst)
    if (allocated(tauv))    deallocate(tauv)
    if (allocated(alphav))  deallocate(alphav)
    if (allocated(iiv))     deallocate(iiv)
    if (allocated(iop))     deallocate(iop)
    if (allocated(iow))     deallocate(iow)
    if (allocated(ihcd))    deallocate(ihcd)
    if (allocated(istore))  deallocate(istore)
    if (allocated(tau1v))   deallocate(tau1v)
    if (allocated(q))       deallocate(q)
    if (allocated(yyrfl))   deallocate(yyrfl)
    if (allocated(ywrk))    deallocate(ywrk)
    if (allocated(ypwrk))   deallocate(ypwrk)
    if (allocated(xc))      deallocate(xc)
    if (allocated(xco))     deallocate(xco)
    if (allocated(du1))     deallocate(du1)
    if (allocated(du1o))    deallocate(du1o)
    if (allocated(gri))     deallocate(gri)
    if (allocated(dgrad2v)) deallocate(dgrad2v)
    if (allocated(ggri))    deallocate(ggri)
    if (allocated(grad2))   deallocate(grad2)
    if (allocated(dffiu))   deallocate(dffiu)
    if (allocated(ddffiu))  deallocate(ddffiu)
    if (allocated(ext))     deallocate(ext)
    if (allocated(eyt))     deallocate(eyt)
  end subroutine dealloc_beam
end module beamdata
